#!/usr/bin/perl

use strict;
use warnings;

use Path::Tiny;
use JSON::XS;
use Text::Hogan::Compiler;
use Text::Wrap qw(fill);

$Text::Wrap::columns   = 999;
$Text::Wrap::separator = '</p><p>';

my $VENDOR_PREFIX = $ENV{'VENDOR_PREFIX'} || '';

my $compiler = Text::Hogan::Compiler->new;

my $data = decode_json path('listing.json')->slurp_utf8;

my $desktop_template
	= $compiler->compile( path('webapp.desktop.in')->slurp_utf8 );
my $appdata_template
	= $compiler->compile( path('webapp.appdata.xml.in')->slurp_utf8 );

foreach my $app ( @{ $data->{'@graph'} } ) {
	my $app_id = $VENDOR_PREFIX . $app->{'@id'};
	my $icon_id
		= ( $app->{icon} and path("hicolor/192x192/$app_id.png")->exists )
		? $app_id
		: 'web-browser';
	my $description        = $app->{description} =~ s{<(http[^\s>]+)>}{$1}gr;
	my $hinted_description = '<p>' . fill( '', '', $description ) . '</p>';

	path("$app_id.desktop")->spew_utf8(
		$desktop_template->render(
			{   ID         => $app_id,
				URL        => $app->{url},
				NAME       => $app->{name},
				ICON       => $icon_id,
				COMMENT    => $app->{comment},
				CATEGORIES => join( ';', @{ $app->{category} } )
					. ';GNOME;GTK;',
			}
		)
	);

	path("$app_id.appdata.xml")->spew_utf8(
		$appdata_template->render(
			{   ID          => $app_id,
				URL         => $app->{url},
				NAME        => $app->{name},
				ICON        => $icon_id,
				COMMENT     => $app->{comment},
				DESCRIPTION => $hinted_description,
				CATEGORIES  => join '',
				map {"<category>$_</category>"} @{ $app->{category} },
			}
		)
	);
}
